# This file was created automatically by SWIG.
import cstrainc
from oof2.SWIG.engine.IO.propertyoutput import PropertyOutputInitPtr
class POInitGeometricStrainPtr(PropertyOutputInitPtr):
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def __repr__(self):
        return "<C POInitGeometricStrain instance at %s>" % (self.this,)
class POInitGeometricStrain(POInitGeometricStrainPtr):
    def __init__(self,*_args,**_kwargs):
        self.this = apply(cstrainc.new_POInitGeometricStrain,_args,_kwargs)
        self.thisown = 1






#-------------- FUNCTION WRAPPERS ------------------



#-------------- VARIABLE WRAPPERS ------------------



#-------------- USER INCLUDE -----------------------

# -*- python -*-
# $RCSfile: cstrain_.py,v $
# $Revision: 1.12 $
# $Author: langer $
# $Date: 2005/08/08 21:39:33 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.engine import fieldindex
from oof2.SWIG.engine import symmmatrix
from oof2.SWIG.engine.IO import propertyoutput
from oof2.common import debug
from oof2.common import enum
from oof2.common import registeredclass
from oof2.common.IO import parameter

class StrainType(registeredclass.RegisteredClass):
    registry = []
    def name(self):
        return self._name
    def iterator(self, planarity):
        return fieldindex.getSymTensorIterator(planarity)
    def newOutputValue(self):
        return symmmatrix.newSymTensorOutputValue()
    tip="Different varieties of strain."
    discussion="""<para>
    <classname>StrainType</classname> objects are used to pick strain
    varieties in <link
    linkend='Section:Output'><classname>Outputs</classname></link>.
    </para>"""
 
class GeometricStrain(StrainType):
    _name="Geometric"

registeredclass.Registration(
    "Geometric Strain",
    StrainType,
    GeometricStrain,
    ordering=0,
    tip="The symmetric derivative of the displacement field.",
    discussion="""
    <para> The geometric strain is the symmetric derivative of the
    <link linkend='Section:Concepts:Displacement'>Displacement</link>
    field.  It depends only on the field, and not on any material
    properties.  </para>
    """)

class ElasticStrain(StrainType):
    _name="Elastic"

registeredclass.Registration(
    "Elastic Strain",
    StrainType,
    ElasticStrain,
    ordering=1,
    tip="The part of the geometric strain that contributes to stress.",
    discussion="""
    <para> The elastic strain is the part of the <link
    linkend='RegisteredClass:GeometricStrain'>geometric strain</link>
    that multiplies the elastic modulus when computing the stress.
    For example, in a system with only elasticity and thermal
    expansion, the elastic strain is the geometric strain minus the
    <link linkend='RegisteredClass:ThermalStrain'>thermal
    strain</link>. </para>
    """)

class ThermalStrain(StrainType):
    _name="Thermal"

registeredclass.Registration(
    "Thermal Strain",
    StrainType,
    ThermalStrain,
    ordering=2,
    tip="Strain induced by thermal expansion",
    discussion="""
    <para> The thermal strain is the <link
    linkend='MenuItem:OOF.Property.Parametrize.Couplings.ThermalExpansion.Isotropic'>
    thermal expansion coefficient</link> times the
    temperature. </para>
    """)

class PiezoelectricStrain(StrainType):
    _name="Piezoelectric"

registeredclass.Registration(
    "Piezoelectric Strain",
    StrainType,
    PiezoelectricStrain,
    ordering=3,
    tip="Strain induced by electric fields in piezoelectric materials",
    discussion="""<para>
    The piezoelectric strain is the piezoelectric tensor times the
    electric field.
    </para>""")



class StrainTypeParameter(parameter.RegisteredParameter):
    def __init__(self, name, value=None, default=None, tip=None):
        parameter.RegisteredParameter.__init__(self, name, StrainType,
                                               value, default, tip)
    def clone(self):
        return StrainTypeParameter(self.name, self.value, self.default,
                                   self.tip)

propertyoutput.SymmMatrix3PropertyOutputRegistration(
    "Strain",
    initializer=POInitGeometricStrain(),
    parameters=StrainTypeParameter("type", default=GeometricStrain(),
                                   tip='The type of strain to compute'),
    tip='Compute a strain',
    discussion="""

    <para>Compute some variety of strain on the &mesh;.  The parameter
    <varname>type</varname> determines exactly which strain is
    computed.</para>
    
    """)
