# -*- python -*-
# $RCSfile: strfunction.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2005/05/23 15:28:13 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common import utils
from oof2.common.IO import parameter
import types
import struct

class StrFunction:
    def __init__(self, arglist, funcstr):
        self.arglist = arglist
        self.funcstr = funcstr
        if funcstr:
            self.function = utils.OOFeval('lambda %s: %s' %
                                          (self.arglist, self.funcstr))
        else:
            self.function = None
    def __call__(self, *args):
        if self.function:
            return self.function(*args)
    def string(self):
        return self.funcstr
    def __repr__(self):
        return "StrFunction('%s', '%s')" % (self.arglist, self.funcstr)
    def __str__(self):                  # used by shortrepr in FieldInit
        return "'%s'" % self.funcstr
    def __eq__(self,other):
        if other:
            if isinstance(other,StrFunction):
                if self.__class__==other.__class__:
                    if self.funcstr==other.funcstr:
                        return 1
        return 0

class XYStrFunction(StrFunction):
    def __init__(self, funcstr):
        StrFunction.__init__(self, 'x, y', funcstr)
    def __repr__(self):
        return "'%s'" % self.funcstr    # shorthand notation


utils.OOFdefine('XYStrFunction', XYStrFunction)
utils.OOFdefine('StrFunction', StrFunction)


class XYStrFunctionParameter(parameter.ParameterBase):
    def __init__(self, name, value=None, default='0.0', tip=None):
        parameter.ParameterBase.__init__(self, name, value, default, tip)
    def checker(self, x):
        if type(x) is types.StringType:
            return
        if isinstance(x, XYStrFunction):
            return
        parameter.raiseTypeError(type(x), 'string or XYStrFunction')
    def set(self, value):
        if type(value) is types.StringType:
            self.__dict__['value'] = XYStrFunction(value)
        elif isinstance(value, XYStrFunction):
            self.__dict__['value'] = value
        elif value is None:
            self.__dict__['value'] = None
        else:
            raise parameter.ParameterMismatch(
                'Got ' + `value` + ' for Parameter ' + self.name)
        self.timestamp.increment()
    def valueDesc(self):
        return 'A string defining a function of x and y.'
    def binaryRepr(self, datafile, value):
        length = len(value.funcstr)
        return struct.pack('>i', length) + value.funcstr
    def binaryRead(self, parser):
        b = parser.getBytes(struct.calcsize('>i'))
        (length,) = struct.unpack('>i', b)
        return XYStrFunction(parser.getBytes(length))
