# -*- python -*-
# $RCSfile: boundarycondPage.py,v $
# $Revision: 1.77 $
# $Author: langer $
# $Date: 2005/08/08 21:39:33 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common import utils
from oof2.common import guitop
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.common.IO.GUI import fileselector
from oof2.engine import profile
from oof2.engine import bdycondition
from oof2.engine.IO import boundarycondition
from oof2.engine.IO import profileIO
from oof2.engine import skeletoncontext
import gobject
import gtk
import oof2.engine.mesh

meshmenu = mainmenu.OOF.Mesh

class BoundaryCondPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Boundary Conditions",
                                 ordering=230,
                                 tip="Set field and flux boundary conditions on the mesh's boundaries.")
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        self.meshwidget = whowidget.WhoWidget(oof2.engine.mesh.meshes,
                                              callback=self.meshCB, scope=self)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)

        middlebox = gtk.HPaned()
        mainbox.pack_start(middlebox, expand=1, fill=1)

        bcbuildframe = gtk.Frame("Condition")
        bcbuildframe.set_shadow_type(gtk.SHADOW_IN)
        middlebox.pack2(bcbuildframe, resize=1, shrink=0)

        # Boundary condition construction stuff here.
        bcbuildbox = gtk.VBox(spacing=2)
        bcbuildframe.add(bcbuildbox)
        bcscroll = gtk.ScrolledWindow()
        bcbuildbox.pack_start(bcscroll, expand=1, fill=1)
        bcscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.bclist = BCList(self) 
        bcscroll.add(self.bclist.gtk)


        bcbuttons = gtk.HBox(homogeneous=True, spacing=2)
        bcbuildbox.pack_start(bcbuttons, expand=0, fill=0)
        
        # Buttons for creating, editing and removing boundary conditions
        self.bcNew = gtk.Button("New...")
        self.bcNew.connect("clicked", self.bcNew_CB)
        guitop.top().tooltip(self.bcNew, "Create a new boundary condition in the current mesh.")
        bcbuttons.pack_start(self.bcNew, expand=1, fill=1)

        self.bcRename = gtk.Button("Rename...")
        self.bcRename.connect('clicked', self.bcRename_CB)
        guitop.top().tooltip(self.bcRename, "Rename the selected boundary condition.")
        bcbuttons.pack_start(self.bcRename, expand=1, fill=1)

        self.bcCopy = gtk.Button("Copy...")
        self.bcCopy.connect('clicked', self.bcCopy_CB)
        guitop.top().tooltip(self.bcCopy, "Copy the selected boundary condition to another mesh.")
        bcbuttons.pack_start(self.bcCopy, expand=1, fill=1)

        self.bcCopyAll = gtk.Button("Copy All...")
        self.bcCopyAll.connect('clicked', self.bcCopyAll_CB)
        guitop.top().tooltip(self.bcCopyAll, "Copy all the boundary conditions to another mesh.")
        bcbuttons.pack_start(self.bcCopyAll, expand=1, fill=1)
        
        self.bcEdit = gtk.Button("Edit...")
        self.bcEdit.connect('clicked', self.bcEdit_CB)
        guitop.top().tooltip(self.bcEdit, "Edit the selected boundary condition.")
        bcbuttons.pack_start(self.bcEdit, expand=1, fill=1)

        self.bcDel = gtk.Button("Delete")
        self.bcDel.connect("clicked", self.bcDel_CB)
        guitop.top().tooltip(self.bcDel, "Remove the selected boundary condition.")
        bcbuttons.pack_start(self.bcDel, expand=1, fill=1)
        
        

        profileframe = gtk.Frame("Profile")
        middlebox.pack1(profileframe, resize=1, shrink=0)
        profileframe.set_shadow_type(gtk.SHADOW_IN)
        profilebox = gtk.VBox(spacing=2)
        profileframe.add(profilebox)

        # Profile construction stuff here.

        pfscroll = gtk.ScrolledWindow()
        profilebox.pack_start(pfscroll, expand=1, fill=1)
        pfscroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.pflist = PFList(self)
        pfscroll.add(self.pflist.gtk)

        profile_top_buttons = gtk.HBox(spacing=2, homogeneous=True)
        profile_bottom_buttons = gtk.HBox(spacing=2, homogeneous=True)
        profilebox.pack_start(profile_top_buttons, expand=0, fill=0)
        profilebox.pack_start(profile_bottom_buttons, expand=0, fill=0)

        self.pfNew = gtk.Button("New...")
        self.pfNew.connect("clicked", self.pfNew_CB)
        guitop.top().tooltip(self.pfNew, "Create a new profile.")
        profile_top_buttons.pack_start(self.pfNew, expand=1, fill=1)

        self.pfRename = gtk.Button("Rename...")
        self.pfRename.connect('clicked', self.pfRename_CB)
        guitop.top().tooltip(self.pfRename, "Rename the selected profile.")
        profile_top_buttons.pack_start(self.pfRename, expand=1, fill=1)

        self.pfCopy = gtk.Button("Copy...")
        self.pfCopy.connect('clicked', self.pfCopy_CB)
        guitop.top().tooltip(self.pfCopy, "Copy the selected profile.")
        profile_top_buttons.pack_start(self.pfCopy, expand=1, fill=1)

        self.pfSave = gtk.Button("Save...")
        self.pfSave.connect('clicked', self.pfSave_CB)
        guitop.top().tooltip(self.pfSave, "Save the selected profile to a file.")
        profile_bottom_buttons.pack_start(self.pfSave, expand=1, fill=1)
        
        self.pfEdit = gtk.Button("Edit...")
        self.pfEdit.connect('clicked', self.pfEdit_CB)
        guitop.top().tooltip(self.pfEdit, "Edit the selected profile.")
        profile_bottom_buttons.pack_start(self.pfEdit, expand=1, fill=1)

        self.pfDel = gtk.Button("Delete")
        self.pfDel.connect("clicked", self.pfDel_CB)
        guitop.top().tooltip(self.pfDel, "Remove the selected profile.")
        profile_bottom_buttons.pack_start(self.pfDel, expand=1, fill=1)

        self.pflist.update()
        self.bclist.update(None)
        # self.sensitize()

        switchboard.requestCallback(("new who", "Skeleton"), self.newSkeleton)
        switchboard.requestCallback(("new who", "Mesh"), self.newMesh)
        switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)

        # Callbacks for when the mesh changes state internally.
        switchboard.requestCallback("boundary conditions changed",
                                    self.newBCs)
        switchboard.requestCallback("profiles changed",
                                    self.newPFs)
        switchboard.requestCallback("profile renamed",
                                    self.PFrenamed)
        switchboard.requestCallback("field defined", self.fieldDefined)
        switchboard.requestCallback("equation activated", self.eqnActivated)
        switchboard.requestCallback("made reservation", self.reservationChanged)
        switchboard.requestCallback("cancelled reservation",
                                    self.reservationChanged)
        switchboard.requestCallback("mesh boundaries changed",
                                    self.newMeshBoundaries)

        
    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        meshname = self.currentSkeletonName()
        if meshname:
            return skeletoncontext.getSkeleton(ms, meshname)

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMesh(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None and \
                 not self.currentMesh().query_reservation()

        if meshok:
            mesh = self.currentMesh()
            neqns = len(mesh.all_equations())
            nflds = len(mesh.all_fields())

        # Always possible to create a new profile.
        self.pfNew.set_sensitive(1)

        bclist = self.bclist.getBCName() is not None
        pflist = self.pflist.getPFName() is not None

        self.bcNew.set_sensitive(meshok and (neqns > 0 or nflds > 0))
        self.bcDel.set_sensitive(bclist and meshok)
        self.bcEdit.set_sensitive(bclist and meshok)
        self.bcCopy.set_sensitive(bclist)
        self.bcCopyAll.set_sensitive(bclist)
        self.bcRename.set_sensitive(bclist and meshok)
        self.pfRename.set_sensitive(pflist)
        self.pfEdit.set_sensitive(pflist)
        self.pfCopy.set_sensitive(pflist)
        self.pfSave.set_sensitive(pflist)
        
        self.bcCopyAll.set_sensitive(len(self.bclist)>0)
        
        current_prof = self.pflist.getPF()
        if current_prof and current_prof.deletable():
            self.pfDel.set_sensitive(1)
        else:
            self.pfDel.set_sensitive(0)
        
    def update(self):
        self.set_state(self.currentFullMeshName())
        
    def set_state(self, meshpath):
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        self.bclist.update(self.currentMesh())
        self.pflist.update()
        mesh = self.currentMesh()
        
    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname): # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()

    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def newMeshBoundaries(self, mesh): # switchboard "mesh boundaries changed"
        if mesh==self.currentMesh():
            self.update()
            self.sensitize()
            
    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()

    def fieldDefined(self, meshname, fieldname, active):
        if meshname == self.currentFullMeshName():
            self.update()
            self.sensitize()

    def eqnActivated(self, meshname, eqnname, active):
        if meshname == self.currentFullMeshName():
            self.update()
            self.sensitize()

    ###############################

    def bcNew_CB(self, gtkobj):         # "New..." button callback
        menuitem = boundarycondition.bcmenu.New
        params = [p for p in menuitem.params if p.name != "mesh"]
        if parameterwidgets.getParameters(title="New Boundary Condition",
                                          scope=self, *params):
            menuitem.callWithDefaults(mesh=self.currentFullMeshName())
        
    def bcDel_CB(self, gtkobj):
        meshname = self.currentFullMeshName()
        bcname = self.bclist.getBCName()
        if meshname and bcname:
            boundarycondition.bcmenu.Delete(mesh=meshname, name=bcname)

    # This one can also copy a BC to another mesh(es).
    def bcCopy_CB(self, gtkobj):
        currentmesh = self.currentFullMeshName()
        bcname = self.bclist.getBCName()
        if currentmesh and bcname:
            menuitem = boundarycondition.bcmenu.Copy
            newnameparam = menuitem.get_arg("name")
            boundaryparam = menuitem.get_arg("boundary")
            # Pre-set the "boundary" parameter.
            bc = oof2.engine.mesh.meshes[currentmesh].getBdyCondition(bcname)
            boundaryparam.set(bc.boundary)
            # 
            meshnameparam = menuitem.get_arg("mesh")
            if parameterwidgets.getParameters(
                newnameparam, boundaryparam, meshnameparam,
                title="Choose a name and boundary.", scope=self):
                menuitem.callWithDefaults(current=currentmesh, bc=bcname)

    # Convenience function to copy all the BCs to another mesh.  
    def bcCopyAll_CB(self, gtkobj):
        currentmesh = self.currentFullMeshName()
        if currentmesh:
            menuitem = boundarycondition.bcmenu.Copy_All
            meshnameparam = menuitem.get_arg("mesh")
            if parameterwidgets.getParameters(
                meshnameparam,
                title="Choose the target mesh.", scope=self):
                menuitem.callWithDefaults(current=currentmesh)
    
    def bcEdit_CB(self, *args):
        meshname = self.currentFullMeshName()
        bcname = self.bclist.getBCName()
        if meshname and bcname:
            menuitem = boundarycondition.bcmenu.Edit
            mesh_ctxt = oof2.engine.mesh.meshes[meshname]

            condition_obj = mesh_ctxt.getBdyCondition(bcname)
            condition_param = menuitem.get_arg('condition')
            # Set the parameter so the dialog box starts with the right object.
            condition_param.set(condition_obj)

            if parameterwidgets.getParameters(
               condition_param, title="Edit Boundary Condition",
               scope=self):
                menuitem.callWithDefaults(name=bcname, mesh=meshname)

    def bcRename_CB(self, gtkobj):
        meshname = self.currentFullMeshName()
        bcname = self.bclist.getBCName()
        if meshname and bcname:
            menuitem = boundarycondition.bcmenu.Rename
            newnameparam = menuitem.get_arg('name')
            newnameparam.value = bcname
            if parameterwidgets.getParameters(newnameparam,
                                              title="Rename the boundary condition\nformerly known as '%s'" % bcname):
                menuitem.callWithDefaults(mesh=meshname, bc=bcname)

    def newBCs(self, mesh, bc, promotion):  # Switchboard callback.
        if mesh==self.currentMesh() and not promotion:
            self.bclist.update(mesh, select=bc)


    ################################

    def pfNew_CB(self, gtkobj):         # button callback
        menuitem = boundarycondition.pfmenu.New
        if parameterwidgets.getParameters(title='New Named Profile',
                                          scope=self, *menuitem.params):
            menuitem.callWithDefaults()
                                    
    def pfDel_CB(self, gtkobj):
        pfname = self.pflist.getPFName()
        if pfname:
            boundarycondition.pfmenu.Delete(name=pfname)
            
    def pfCopy_CB(self, gtkobj):
        profname = self.pflist.getPFName()
        if profname:
            menuitem = boundarycondition.pfmenu.Copy
            newnameparam = menuitem.get_arg('name')
            newnameparam.value = profname
            if parameterwidgets.getParameters(newnameparam,
                                              title="Name the copied profile"):
                menuitem.callWithDefaults(profile=profname)

    def pfEdit_CB(self, *args):
        profname = self.pflist.getPFName()
        if profname:
            # Set the initial value of the profile parameter to be the
            # profile in question.
            menuitem = boundarycondition.pfmenu.Edit
            profile_obj = profile.AllProfiles[profname]
            profile_param = menuitem.get_arg('profile')
            profile_param.value = profile_obj
            if parameterwidgets.getParameters(
                profile_param, title="Edit profile."):
                menuitem.callWithDefaults(name=profname)
                                                               
    def pfSave_CB(self, *args):
        profname = self.pflist.getPFName()
        if profname:
            menuitem = profileIO.profile_save
            file = fileselector.fileSelector(
                ident="ProfileSaveFromPage",
                title="Save Profile",
                params=(menuitem.get_arg('format'),) )
            if file is not None:
                mode = fileselector.get_file_mode(file)
                if mode is not None:
                    menuitem.callWithDefaults(filename=file,
                                              mode=mode,
                                              profile=profname)

    # Similar in operation to pfCopy_CB.
    def pfRename_CB(self, gtkobj):
        profname = self.pflist.getPFName()
        if profname:
            menuitem = boundarycondition.pfmenu.Rename
            newnameparam = menuitem.get_arg('name')
            newnameparam.value = profname
            if parameterwidgets.getParameters(newnameparam,
                                              title="Rename the profile"):
                menuitem.callWithDefaults(profile=profname)
            
    # Because the displayed BC's display the names of profiles, both
    # lists need to be updated when the pflist changes.
    def newPFs(self, name): # Switchboard callback.
        self.pflist.update(select=name)


    # Name arguments discarded here.  When a profile is renamed, both
    # lists have to be updated, because BC descriptors refer to their
    # profiles.
    def PFrenamed(self, oldname, newname):
        self.pflist.update(select=newname)
        self.bclist.redraw()



# Stateful widget for listing the currently-available boundary conditions.
class BCList:
    def __init__(self, parent, mesh=None):
        self.parent = parent # Reference to the enclosing BoundaryCondPage.
        self.current_mesh = mesh
        debug.mainthreadTest()
        self.bcliststore = gtk.ListStore(gobject.TYPE_STRING,
                                         gobject.TYPE_PYOBJECT)
        self.gtk = gtk.TreeView(self.bcliststore)
        self.gtk.set_headers_clickable(False)

        bcnamecell = gtk.CellRendererText()
        bcnamecol = gtk.TreeViewColumn('Name')
        bcnamecol.pack_start(bcnamecell, expand=False)
        bcnamecol.set_attributes(bcnamecell, text=0)
        self.gtk.append_column(bcnamecol)

        bccell = gtk.CellRendererText()
        bccol = gtk.TreeViewColumn('Condition')
        bccol.pack_start(bccell, expand=True)
        bccol.set_cell_data_func(bccell, self.renderBC)
        self.gtk.append_column(bccol)

        self.gtk.get_selection().connect('changed', self.selectCB)
        self.gtk.connect('row-activated', self.doubleClickCB)
        
##        self.gtk = gtk.CList(cols=2,titles=["Name","Condition"])
##        self.gtk.set_selection_mode(SELECTION_SINGLE)
##        self.gtk.set_column_auto_resize(0,1)
##        self.gtk.set_column_auto_resize(1,1)
##        self.gtk.column_titles_passive()
##        self.select = self.gtk.connect("select-row", self.selectCB)
##        self.unselect = self.gtk.connect("unselect-row", self.unselectCB)
##        self.gtk.connect("button_press_event", self.buttonpressCB)
##        self.current_name = None # This will be a name.
##        self.names = []
##        self.dbclicked = 0

    def renderBC(self, column, cell_renderer, model, iter):
        bc = model[iter][1]
        cell_renderer.set_property('text', bc.display_string())

    def getBC(self):
        selection = self.gtk.get_selection()
        model, iter = selection.get_selected()
        if iter:
            return model[iter][1]
    def getBCName(self):
        selection = self.gtk.get_selection()
        model, iter = selection.get_selected()
        if iter:
            return model[iter][0]
                                   
    # Some profile-driven updates don't have a mesh -- use this
    # call to use the current mesh, if any.
    def redraw(self):
        self.update(self.current_mesh)

    def __len__(self):
        return len(self.bcliststore)
    
    # Simplest to just rebuild the whole list.
    def update(self, mesh, select=None):
        debug.mainthreadTest()
        self.current_mesh = mesh
        currentBC = select or self.getBCName()
        self.bcliststore.clear()
        if mesh:
            items = mesh.allBoundaryConds()[:]
            items.sort()
            for (name, bc) in items:
                # Promoted floatBCs should not be listed.
                if not bc.promotion():
                    self.bcliststore.append([name, bc])
            if currentBC is not None:
                for row in range(len(self.bcliststore)):
                    if self.bcliststore[row][0] == currentBC:
                        self.gtk.get_selection().select_path(row)
                        break
        self.parent.sensitize()

##    def selectCB(self, gtkobj, row, col, event):
##        self.current_name = self.names[row]
##        self.parent.sensitize()
##        if self.dbclicked:
##            self.dbclicked = 0
##            self.parent.bcEdit_CB()
    def selectCB(self, selection):
        self.parent.sensitize()
    def doubleClickCB(self, treeview, path, col):
        self.parent.bcEdit_CB()
            
##    def unselectCB(self, gtkobj, row, col, event):
##        self.current_name = None
##        self.parent.sensitize()

##    def buttonpressCB(self, gtkwidget, gdkevent):
##        if gdkevent.type == GDK._2BUTTON_PRESS:
##            self.dbclicked = 1
            
##    def getBC(self):
##        if self.current_mesh:
##            if self.current_name:
##                return self.current_mesh.getBdyCondition(self.current_name)
##        return None

##    def getBCName(self):
##        if self.current_mesh:
##            return self.current_name
##        return None
            

# Similar one for boundary condition profiles.  "PF" may not
# be the most descriptive name.
class PFList:
    def __init__(self, parent):
        self.parent = parent
        debug.mainthreadTest()
        self.pfliststore = gtk.ListStore(gobject.TYPE_STRING,
                                         gobject.TYPE_PYOBJECT)
        self.gtk = gtk.TreeView(self.pfliststore)
        self.gtk.set_headers_clickable(False)

        pfnamecell = gtk.CellRendererText()
        pfnamecol = gtk.TreeViewColumn('Name')
        pfnamecol.pack_start(pfnamecell, expand=False)
        pfnamecol.set_attributes(pfnamecell, text=0)
        self.gtk.append_column(pfnamecol)

        pfcell = gtk.CellRendererText()
        pfcol = gtk.TreeViewColumn('Profile')
        pfcol.pack_start(pfcell, expand=True)
        pfcol.set_cell_data_func(pfcell, self.renderPF)
        self.gtk.append_column(pfcol)

        self.gtk.get_selection().connect('changed', self.selectCB)
        self.gtk.connect('row-activated', self.doubleClickCB)
##        self.gtk = gtk.CList(cols=2,titles=["Name","Profile"])
##        self.gtk.set_selection_mode(SELECTION_SINGLE)
##        self.gtk.set_column_auto_resize(0,1)
##        self.gtk.set_column_auto_resize(1,1)
##        self.gtk.column_titles_passive()
##        self.select = self.gtk.connect("select-row", self.selectCB)
##        self.unselect = self.gtk.connect("unselect-row", self.unselectCB)
##        self.gtk.connect("button_press_event", self.buttonpressCB)
##        self.current_name = None # This will be a name.
##        self.names = []
##        self.dbclicked = 0

    def renderPF(self, column, cell_renderer, model, iter):
        prof = model[iter][1]
        cell_renderer.set_property('text', `prof`)

    # Simplest to just rebuild the whole list.
    def update(self, select=None):
        debug.mainthreadTest()
        currentProf = select or self.getPFName()
        self.pfliststore.clear()
        items = profile.AllProfiles.items()
        items.sort()
        for name, prof in items:
            self.pfliststore.append([name, prof])
        if currentProf is not None:
            for row in range(len(self.pfliststore)):
                if self.pfliststore[row][0] == currentProf:
                    self.gtk.get_selection().select_path(row)
                    break
##        self.gtk.clear()
##        self.names=[]
##        items = profile.AllProfiles.items()[:]
##        items.sort()
##        for (name, prof) in items:
##            self.names.append(name)
##            self.gtk.append([name, `prof`])
##        try:
##            if select is not None:
##                self.current_name = select
##            self.gtk.select_row(self.names.index(self.current_name), 0)
##        except ValueError:
##            self.current_name=None
##            self.gtk.unselect_all()
        self.parent.sensitize()

    def selectCB(self, selection):
##        self.current_name = self.names[row]
        self.parent.sensitize()
##        if self.dbclicked:
##            self.dbclicked = 0
##            self.parent.pfEdit_CB()

    def doubleClickCB(self, treeview, path, col):
        self.parent.pfEdit_CB()
            
##    def unselectCB(self, gtkobj, row, col, event):
##        self.current_name = None
##        self.parent.sensitize()

##    def buttonpressCB(self, gtkwidget, gdkevent):
##        if gdkevent.type == GDK._2BUTTON_PRESS:
##            self.dbclicked = 1

    def getPF(self):
        selection = self.gtk.get_selection()
        model, iter = selection.get_selected()
        if iter:
            return model[iter][1]
##        if self.current_name:
##            return profile.AllProfiles[self.current_name]
##        return None

    def getPFName(self):
        selection = self.gtk.get_selection()
        model, iter = selection.get_selected()
        if iter:
            return model[iter][0]
##        return self.current_name


# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
        
bcp = BoundaryCondPage()
