# -*- python -*-
# $RCSfile: meshinfoGUI.py,v $
# $Revision: 1.73 $
# $Author: langer $
# $Date: 2005/08/08 21:39:34 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import devel
from oof2.SWIG.common import switchboard
from oof2.SWIG.engine import planarity
from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import primitives
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import toolboxGUI
from oof2.engine.IO import meshinfo
from oof2.engine.IO.GUI import meshdataGUI
import gtk
import oof2.engine.mesh
import oof2.engine.subproblemcontext
import string
import math

xpadding = 3
xoptions = gtk.EXPAND|gtk.FILL

## TODO LATER: Can SkeletonToolboxGUI and MeshToolboxGUI share a base
## class?

class MeshInfoMode:
    # Base class for ElementMode, NodeMode.
            
    def __init__(self, toolbox):
        debug.mainthreadTest()
        self.toolbox = toolbox  # MeshToolboxGUI
        self.menu = self.toolbox.toolbox.menu  # ie, gfxtoolbox.toolbox.menu

        self.gtk = gtk.Frame(self.targetname + " Information")
        self.gtk.set_shadow_type(gtk.SHADOW_IN)
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.gtk.add(scroll)
        # This vbox just keeps the table from expanding inside the
        # scrolledwindow.
        vbox = gtk.VBox()
        scroll.add_with_viewport(vbox)
        self.table = gtk.Table(rows=self.nrows, columns=self.ncols)
        vbox.pack_start(self.table, expand=0, fill=0)

    def destroy(self):
        mainthread.runBlock(self.gtk.destroy)
        self.toolbox = None  # break circular references

    def clearQuery(self):
        self.updateNothing()
            
    # Utility routines for constructing GUI components
    
    def labelmaster(self, column, row, labeltext):
        debug.mainthreadTest()
        label = gtk.Label(labeltext)
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)

    def entrymaster(self, column, row, editable=0):
        debug.mainthreadTest()
        entry = gtk.Entry()
        entry.set_size_request(12*guitop.top().digitsize, -1)
        entry.set_editable(editable)
        self.table.attach(entry, column[0],column[1], row[0],row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return entry

    def getNodeCB(self, node):
        self.toolbox.changeModeWithObject(node, "Node")

    def showNodeCB(self, node):
        if node:
            self.toolbox.toolbox.peeker.assignObject(node, "Node")
        else:
            self.toolbox.toolbox.peeker.removeObject("Node")
        switchboard.notify("redraw")

    def makeNodeList(self, column, row):
        chsr = chooser.FramedChooserListWidget(callback=self.showNodeCB,
                                               dbcallback=self.getNodeCB,
                                               autoselect=0)
        self.table.attach(chsr.gtk, column[0], column[1], row[0], row[1],
                          xpadding=xpadding, xoptions=xoptions)
        return chsr
    def updateNodeList(self, chsr, nodes):
        namelist = ["%s %d at (%g, %g)" % 
                   (node.classname(), node.index(),
                    node.position().x, node.position().y)
                    for node in nodes]
        chsr.update(nodes, namelist)
##    def menumasterNode(self, column, row, objlist=[]):
##        debug.mainthreadTest()
##        namelist = [None]*len(objlist)
##        for i in range(len(objlist)):
##            obj = objlist[i]
##            name = "%s %d at (%g, %g)" % \
##                   (obj.classname(), obj.index(),
##                    obj.position().x, obj.position().y)
##            namelist[i] = name
##        menu = chooser.FramedChooserListWidget(
##            objlist=objlist, displaylist=namelist, callback=self.showNodeCB,
##            dbcallback=self.getNodeCB, autoselect=None)
##        self.table.attach(menu.gtk, column[0],column[1], row[0],row[1],
##                          xpadding=xpadding, xoptions=xoptions)
##        return menu

######################
    
class ElementMode(MeshInfoMode):
    nrows=4
    ncols=2
    targetname = "Element"
    def __init__(self, toolbox, querier=None):
        MeshInfoMode.__init__(self, toolbox)

        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))

        self.labelmaster((0,1), (1,2), 'type=')
        self.type = self.entrymaster((1,2), (1,2))

        self.labelmaster((0,1), (2,3), 'nodes=')
        self.nodes = self.makeNodeList((1,2), (2,3))

        self.labelmaster((0,1), (3,4), 'material=')
        self.material = self.entrymaster((1,2), (3,4))

    def menucmd(self):
        return self.menu.QueryElement
            
    def updateSomething(self, container):
        debug.mainthreadTest()
        element = container.object
        femesh = container.context.femesh()
        coord = container.position
        
        self.type.set_text(element.masterelement().name())
        self.index.set_text(`element.get_index()`)

        ## We can't just pass element.node_iterator() in to
        ## updateNodeList() because the iterator doesn't quite act
        ## like a list.  The ChooserListWidget uses list.index(),
        ## which the iterator doesn't have.
        self.updateNodeList(self.nodes,
                            [node for node in element.node_iterator()])

        mat = element.material()
        if mat:
            self.material.set_text(mat.name())
        else:
            self.material.set_text("<No material>")

    def updateNothing(self):
        debug.mainthreadTest()
        self.type.set_text("")
        self.index.set_text("")
        
##        self.nodes.destroy()
##        self.nodes = self.menumasterNode((1,2), (2,3))
        self.nodes.update([])
        self.material.set_text("")

######################
    
class NodeMode(MeshInfoMode):
    targetname = "Node"
    nrows = 3
    ncols = 2
    def __init__(self, toolbox, querier=None):
        MeshInfoMode.__init__(self, toolbox)

        self.labelmaster((0,1), (0,1), 'index=')
        self.index = self.entrymaster((1,2), (0,1))

        self.labelmaster((0,1), (1,2), 'type=')
        self.type = self.entrymaster((1,2), (1,2))

        self.labelmaster((0,1), (2,3), 'position=')
        self.pos = self.entrymaster((1,2), (2,3))

    def updateSomething(self, container):
        debug.mainthreadTest()
        node = container.object
        femesh = container.context.femesh()
        
        self.index.set_text(`node.index()`)
        self.type.set_text(node.classname())
        self.pos.set_text("(%g, %g)" %
                          (node.position().x, node.position().y))

    def updateNothing(self):
        debug.mainthreadTest()
        self.index.set_text("")
        self.type.set_text("")
        self.pos.set_text("")
        
    def menucmd(self):
        return self.menu.QueryNode


############################################################################

modes = [ElementMode, NodeMode]

############################################################################
############################################################################

class MeshToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, meshinfo):
        toolboxGUI.GfxToolbox.__init__(self, "Mesh Info", meshinfo)
        self.mainbox = gtk.VBox()
        self.gtk.add(self.mainbox)

        self.modeclass = ElementMode
        self.modeobj = None
        self.modeobjdict = {}

        clickframe = gtk.Frame()
        clickframe.set_shadow_type(gtk.SHADOW_IN)
        self.mainbox.pack_start(clickframe, expand=0, fill=0)
        clickbox = gtk.VBox()
        clickframe.add(clickbox)

        hbox = gtk.HBox()
        clickbox.pack_start(hbox, expand=0, fill=0)
        hbox.pack_start(gtk.Label("Click on an: "), expand=0, fill=0)

        self.modebuttons = []
        self.modebuttondict = {}
        for mode in modes:
            if self.modebuttons:
                button = gtk.RadioButton(label=mode.targetname,
                                        group=self.modebuttons[0])
            else:
                button = gtk.RadioButton(label=mode.targetname)
            self.modebuttons.append(button)
            guitop.top().tooltip(button,
                               "Show " + mode.targetname + "information")
            hbox.pack_start(button, expand=0, fill=0)
            button.set_active(self.modeclass is mode)
            button.connect('clicked', self.changeModeCB, mode)
            self.modebuttondict[mode.targetname] = button
                        
        # Display mouse click coordinates
        self.table = gtk.Table(columns=2, rows=3)
        clickbox.pack_start(self.table, expand=0, fill=0)

        label = gtk.Label('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,1, xpadding=5, xoptions=gtk.FILL)
        self.xtext = gtk.Entry()
        self.xtext.set_editable(0)
        self.xtext.set_size_request(13*guitop.top().charsize, -1)
        self.table.attach(self.xtext, 1,2, 0,1,
                          xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)
        label = gtk.Label('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 1,2, xpadding=5, xoptions=gtk.FILL)
        self.ytext = gtk.Entry()
        self.ytext.set_size_request(13*guitop.top().charsize, -1)
        self.ytext.set_editable(0)
        self.table.attach(self.ytext, 1,2, 1,2,
                          xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)
        guitop.top().tooltip(self.xtext, "x coordinate of the mouse click")
        guitop.top().tooltip(self.ytext, "y coordinate of the mouse click")

        # End of clicked point display

        self.infoframe = gtk.Frame()
        self.infoframe.set_shadow_type(gtk.SHADOW_NONE)
        self.mainbox.pack_start(self.infoframe, expand=1, fill=1)
        self.meshcontext = apply(self.toolbox.gfxwindow().topwho, ('Mesh',))
        self.buildInfoGUI(self.modeclass)
        # End of query data display

        align = gtk.Alignment(xalign=0.5)
        self.mainbox.pack_start(align, expand=0, fill=0, padding=2)
        centerbox = gtk.HBox()
        align.add(centerbox)
        self.meshdatabutton = gtk.Button('New Data Viewer...')
        centerbox.pack_start(self.meshdatabutton, expand=0, fill=0)
        self.meshdatabutton.connect('clicked', self.meshdataCB)
        guitop.top().tooltip(self.meshdatabutton,
                             "Open a window to display fields, fluxes, and other quantities evaluated at the mouse click location.")

        buttonbox = gtk.HBox()
        self.mainbox.pack_start(buttonbox, expand=0, fill=0)
        self.prev = gtk.Button()
        lbox = gtk.HBox()
        lbox.add(gtk.Arrow(gtk.ARROW_LEFT, gtk.SHADOW_OUT))
        lbox.add(gtk.Label(' Prev'))
        guitop.top().tooltip(self.prev, "Go back to the previous object.")
        self.prev.add(lbox)
        self.prev.connect('clicked', self.prevQuery)
        buttonbox.pack_start(self.prev, expand=0, fill=0, padding=2)
        
        self.clear = gtk.Button("Clear")
        self.clear.connect('clicked', self.clearQuery)
        guitop.top().tooltip(self.clear, "Clear the current query.")
        buttonbox.pack_start(self.clear, expand=1, fill=1, padding=2)

        self.next = gtk.Button()
        lbox = gtk.HBox()
        lbox.add(gtk.Label('Next '))
        guitop.top().tooltip(self.next, "Go to the next object.")
        lbox.add(gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_OUT))
        self.next.add(lbox)
        self.next.connect('clicked', self.nextQuery)
        buttonbox.pack_start(self.next, expand=0, fill=0, padding=2)

        self.mainbox.show_all()
        self.sensitize()

        self.xposition = None
        self.yposition = None
        
        self.sbcallbacks = [
            switchboard.requestCallback((self.toolbox.gfxwindow(),"query mesh"),
                                        self.updateQuery),
            switchboard.requestCallback((self.toolbox.gfxwindow(),
                                         "layers changed"),
                                        self.newLayers),
            switchboard.requestCallback("field activated",
                                        self.fieldFluxChanged),
            switchboard.requestCallback("field defined", self.fieldFluxChanged),
            switchboard.requestCallback("equation activated",
                                        self.fieldFluxChanged),
            switchboard.requestCallback("mesh changed", self.meshChanged)
            ]
        
    def buildInfoGUI(self, modeclass):
        debug.mainthreadTest()
        if self.modeobj and self.modeobj.__class__ is modeclass:
            return
        if self.modeobj:
            self.infoframe.remove(self.modeobj.gtk)
        # Reuse existing modeobj if possible
        try:
            self.modeobj = self.modeobjdict[modeclass]
        except KeyError:
            self.modeobj = self.modeobjdict[modeclass] = \
                           modeclass(self)
        self.infoframe.add(self.modeobj.gtk)

    def updateQuery(self):
        if not self.toolbox.querier:
            return
        # See if there's anything to update
        if self.toolbox.querier.object:
            # Change mode if needed.
            self.handleMode(self.toolbox.querier.targetname)
            querier = self.toolbox.querier
            self.modeobj.updateSomething(querier)
            self.showPosition(querier.position.x, querier.position.y)
        else:
            self.modeobj.updateNothing()
        self.sensitize()

    def newLayers(self, *args):
        self.meshcontext = apply(self.toolbox.gfxwindow().topwho, ('Mesh',))
        if self.meshcontext is not None:
            self.buildInfoGUI(self.modeclass)
            self.mainbox.show_all()
            self.updateQuery()
            self.sensitize()
        else:
            self.modeobj.clearQuery()
            self.sensitize()

    def fieldFluxChanged(self, meshname, *args):
        if devel.devel():
            try:
                sbpblem = oof2.engine.subproblemcontext.subproblems[meshname]
                femesh = sbpblem.getMesh()
            except KeyError:
                try:
                    femesh = oof2.engine.mesh.meshes[meshname]
                except KeyError, excp:
                    raise excp
        else:
            femesh = oof2.engine.mesh.meshes[meshname]
        if self.meshcontext and self.meshcontext is femesh:
            self.buildInfoGUI(self.modeclass)
            self.mainbox.show_all()
            self.updateQuery()
            self.sensitize()
    def meshChanged(self, meshcontext):
        if self.meshcontext is meshcontext:
            self.buildInfoGUI(self.modeclass)
            self.mainbox.show_all()
            self.updateQuery()
            self.sensitize()

    def sensitize(self):
        debug.mainthreadTest()
        self.clear.set_sensitive(self.clearable())
        self.prev.set_sensitive(self.prev_able())
        self.next.set_sensitive(self.next_able())

    def clearable(self):
        try:
            return self.toolbox.querier.clearable()
        except AttributeError:
            return 0

    def prev_able(self):
        return self.toolbox.prev_able()

    def next_able(self):
        return self.toolbox.next_able()

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.toolbox.resetRecords()
        self.sensitize()

    def close(self):
        if self.modeobj:
            self.modeobj.destroy()
        map(switchboard.removeCallback, self.sbcallbacks)
        toolboxGUI.GfxToolbox.close(self)

    def acceptEvent(self, eventtype):
        return eventtype == 'up'

    def up(self, x, y, shift, ctrl):
        self.modeobj.menucmd()(position=primitives.Point(x,y))

    def changeMode(self, mode):            
        self.modeclass = mode
        self.buildInfoGUI(mode)
        self.mainbox.show_all()
        self.toolbox.resetPeeker()

        # Automatically query object with a current mouse-click point
        if self.xposition is not None and self.yposition is not None:
            self.modeobj.menucmd()(position=primitives.Point(self.xposition,
                                                             self.yposition))

    def changeModeCB(self, button, mode):
        if self.modeclass is not mode:
            self.changeMode(mode)

    def blockAutoQuery(self):
        self.xposition = None
        self.yposition = None

    def changeModeWithObject(self, object, objtype):
        self.blockAutoQuery()
        self.modebuttondict[objtype].set_active(1)
        if objtype == "Element":
            self.modeobj.menucmd()(position=object.center())
        elif objtype == "Node":
            self.modeobj.menucmd()(position=object.position())
        
    def showPosition(self, x, y):
        debug.mainthreadTest()
        self.xtext.set_text("%-13.6g" % x)
        self.ytext.set_text("%-13.6g" % y)
        self.xposition = x
        self.yposition = y

    def prevQuery(self, gtkobj):
        prev = self.toolbox.records.prev().clone()
        targetname = prev.targetname
        self.handleMode(targetname)
        self.toolbox.querier = prev
        self.toolbox.querysignal()

    def clearQuery(self, gtkobj):
        self.modeobj.clearQuery()
        self.toolbox.clearQuerier()
        
    def nextQuery(self, gtkobj):
        next = self.toolbox.records.next().clone()
        targetname = next.targetname
        self.handleMode(targetname)
        self.toolbox.querier = next
        self.toolbox.querysignal()

    def handleMode(self, targetname):
        for mode in modes:
            if mode.targetname==targetname:
                if self.modeclass is not mode:
                    self.blockAutoQuery()
                    self.modebuttons[modes.index(mode)].set_active(1)
                break

    def meshdataCB(self, button):
        meshdataGUI.openMeshData(self.gfxwindow(), self.toolbox.currentPoint())
                              

def _makeGUI(self):
    return MeshToolboxGUI(self)

meshinfo.MeshInfoToolbox.makeGUI = _makeGUI
