# -*- python -*-
# $RCSfile: meshbdymenu.py,v $
# $Revision: 1.10 $
# $Author: langer $
# $Date: 2005/05/13 18:44:09 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Menu items for post-solution actions on boundaries.
from oof2.SWIG.engine.IO import bdyflux
from oof2.common.IO import automatic
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO import destinationparam
from oof2.common.IO import whoville
from oof2.common.IO import xmlmenudump
from oof2.engine.IO import meshmenu
from oof2.engine.IO import meshparameters
#from oof2.engine.IO import skeletongroupparams # For boundary param class.
import oof2.engine.mesh

meshmenutop = meshmenu.meshmenu

meshbdymenu = meshmenutop.addItem(oofmenu.OOFMenuItem(
    'Boundary',
    secret=1,
    help="Post-solution actions on boundaries.",
    discussion="""<para>
    The <command>OOF.Mesh.Boundary</command> menu contains analysis
    commands that operate on the &mesh; boundaries.  Compare to <xref
    linkend='MenuItem:OOF.Mesh.Analyze'/>, which contains commands
    that operate on the interior of the &mesh;.
    </para>"""
    ))

# "mesh" and "boundary" are strings, "flux" is an object, and
# "destination" is a filename or "automatic".
def _integrateFlux(menuitem, mesh, boundary, flux, destination):
    fe_mesh = oof2.engine.mesh.meshes[mesh].getObject()
    edgeset = fe_mesh.getBoundary(boundary).edgeset
    outfile = destinationparam.get_file(destination)
    res = bdyflux.integrate(fe_mesh, flux, edgeset)
    print >> outfile, res

meshbdymenu.addItem(oofmenu.OOFMenuItem(
    'Integrate_Flux',
    callback=_integrateFlux,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            meshparameters.MeshEdgeBdyParameter('boundary',
                                                tip="Target boundary."),
            meshparameters.FluxParameter('flux', tip=parameter.emptyTipString),
            destinationparam.DestinationParameter('destination',
                                                  value=automatic.automatic,
                                                 tip="Where to put the result?")
            ],
    help="Integrate a Flux over a specified boundary.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/menu/integrate_flux.xml')
    ))
