# -*- python -*-
# $RCSfile: anneal.py,v $
# $Revision: 1.47 $
# $Author: langer $
# $Date: 2005/05/13 18:44:03 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import debug
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import reporter
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump
from oof2.engine import fiddlenodesbase
from oof2.engine import skeletonmodifier
from oof2.SWIG.common.IO import progressbar
from oof2.SWIG.common import switchboard
import random

FiddleNodes = fiddlenodesbase.FiddleNodes
FiddleNodesTargets = fiddlenodesbase.FiddleNodesTargets
IterationManager = fiddlenodesbase.IterationManager

class AnnealMovePosition(fiddlenodesbase.FiddleNodesMovePosition):
    def __init__(self, delta):
        self.delta = delta
    def __call__(self, skeleton, node):
        px, py = skeleton.MS.sizeOfPixels()
        dx = random.gauss(0.0, self.delta*px)
        dy = random.gauss(0.0, self.delta*py)
        return primitives.Point(node.position()[0]+dx,
                                node.position()[1]+dy)

registeredclass.Registration(
    'Anneal Move Position', fiddlenodesbase.FiddleNodesMovePosition,
    AnnealMovePosition,
    ordering=0,
    params = [parameter.FloatParameter("delta", 1.0, tip='width of gaussian distribution of node motions, in pixel units')],)

class Anneal(FiddleNodes, skeletonmodifier.SkeletonModifier):
    def __init__(self, targets, criterion, T, delta, iteration):
        FiddleNodes.__init__(self, targets, criterion, T, iteration)
        self.delta = delta
        self.intro = "preparing to anneal skeleton...      "
        self.header = "Annealing skeleton: "
        self.outro = "Annealing done: "
        self.movedPosition = AnnealMovePosition(self.delta)
        
    ## def movedPosition(self, skeleton, node):
    ##    px, py = skeleton.MS.sizeOfPixels()
    ##    dx = random.gauss(0.0, self.delta*px)
    ##    dy = random.gauss(0.0, self.delta*py)
    ##    return primitives.Point(node.position()[0]+dx,
    ##                            node.position()[1]+dy)

#################################################

registeredclass.Registration(
    'Anneal',
    skeletonmodifier.SkeletonModifier,
    Anneal,
    ordering=3,
    params=[parameter.RegisteredParameter('targets', FiddleNodesTargets,
                                          tip='Which nodes to move.'),
            parameter.RegisteredParameter('criterion',
                                          skeletonmodifier.SkelModCriterion,
                                          tip = 'Acceptance criterion'),
            parameter.FloatParameter('T', value = 0.0,
                                     tip='Failed moves will be accepted if T>0 and exp(-diffE/T) > r, where diffE is the energy gained and r is a random number between 0 and 1.'),
            parameter.FloatParameter('delta', value=1.0,
                                     tip='Width of the distribution of attempted node motions, in units of the pixel size.'),
            parameter.RegisteredParameter('iteration', IterationManager,
                                          tip='Iteration method.')
    ],
    tip='Move nodes randomly and accept the ones that meet the acceptance criterion.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg/anneal.xml')
    )
