# -*- python -*-
# $RCSfile: field_.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2005/05/13 18:44:04 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

import types
from oof2.common import debug
from oof2.common import utils
from oof2.engine import component
from oof2.SWIG.engine import fieldindex

FieldPtr.__repr__ = FieldPtr.name
CompoundFieldPtr.__repr__ = CompoundFieldPtr.name
ScalarFieldPtr.__repr__ = ScalarFieldPtr.name
TwoVectorFieldPtr.__repr__ = TwoVectorFieldPtr.name

#-----------------

def _Field___cmp__(self, other):
    if not isinstance(other, FieldPtr): return -1
    if self.objectid() < other.objectid(): return -1
    if self.objectid() > other.objectid(): return 1
    return 0

FieldPtr.__cmp__ = _Field___cmp__

def _Field___hash__(self):
    return self.objectid()

FieldPtr.__hash__ = _Field___hash__

################

# Fields aren't subclasses of PythonExportable because
# PythonExportable doesn't work properly with the Fields' complicated
# virtual inheritance structure.  That means that Fields have to
# exported as generic Field or CompoundField objects.  But since
# Fields are created initially in Python, and are injected into the
# OOF namespace, we can just use their names to retrieve the Python
# variable and bypass all of the PythonExportable stuff.

def _Field_export(self):
    return utils.OOFeval(self.name())

FieldPtr.export = _Field_export

################

# AllFields acts like a list of CompoundFields.

class _AllFields:
    def __getitem__(self, indx):
        if type(indx) == types.IntType:
            if indx >= countCompoundFields() or indx < 0:
                raise IndexError("Bad field index: " + str(indx))
            return getCompoundFieldByIndex(indx).export()
        elif type(indx) == types.StringType:
            try:
                fld = utils.OOFeval(indx)
            except NameError:
                raise IndexError("Bad field name: " + indx)
            if not isinstance(fld, FieldPtr):
                raise IndexError("Bad field name: " + indx)
            return fld
    def __len__(self):
        return countCompoundFields()

allFields = _AllFields()

def getCompoundField(name):
    return allFields[name]

from oof2.SWIG.engine.freedom import DegreeOfFreedomPtr

########################################

def _Field___getitem__(self, comp):
    # deFortranify
    if type(comp) is types.TupleType:
        c = [x-1 for x in comp]
    else:
        c = [comp-1]
    return component.Component(self, c)


FieldPtr.__getitem__ = _Field___getitem__
