// -*- C++ -*-
// $RCSfile: outputval.C,v $
// $Revision: 1.7 $
// $Author: langer $
// $Date: 2005/05/13 18:44:06 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#include "engine/fieldindex.h"
#include "engine/outputval.h"
#include <math.h>

std::string OutputVal::modulename_("oof2.SWIG.engine.outputval");
std::string ScalarOutputVal::classname_("ScalarOutputVal");
std::string VectorOutputVal::classname_("VectorOutputVal");

OutputValue::OutputValue(OutputVal *v)
  : val(v)
{
  ++val->refcount;
}

OutputValue::OutputValue(const OutputValue &other)
  : val(other.val)
{
  ++val->refcount;
}

OutputValue::~OutputValue() {
  if(--val->refcount == 0) {
    delete val;
  }
}

OutputValue operator*(double x, const OutputValue &ov) {
  OutputValue result(ov);
  result *= x;
  return result;
}

OutputValue operator*(const OutputValue &ov, double x) {
  OutputValue result(ov);
  result *= x;
  return result;
}

OutputValue operator+(const OutputValue &a, const OutputValue &b) {
  OutputValue result(a);
  result += b;
  return result;
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

double ScalarOutputVal::operator[](const IndexP&) const {
  return val;
}

double &ScalarOutputVal::operator[](const IndexP&) {
  return val;
}

IndexP ScalarOutputVal::getIndex(const std::string&) const {
  return IndexP(new ScalarFieldIndex());
}

IteratorP ScalarOutputVal::getIterator() const {
  return IteratorP(new ScalarFieldIterator());
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

VectorOutputVal::VectorOutputVal(int n)
  : data(new double[n]),
    size_(n)
{
  for(int i=0; i<n; i++)
    data[i] = 0.0;
}

double VectorOutputVal::operator[](const IndexP &p) const {
  return data[p.integer()];
}
  
double &VectorOutputVal::operator[](const IndexP &p) {
  return data[p.integer()];
}

IndexP VectorOutputVal::getIndex(const std::string &str) const {
  // str must be "x", "y", or "z"
  return IndexP(new VectorFieldIndex(str[0] - 'x'));
}

IteratorP VectorOutputVal::getIterator() const {
  return IteratorP(new VectorFieldIterator(0, size_));
}

double VectorOutputVal::magnitude() const {
  double sum = 0.0;
  for(int i=0; i<size_; i++) {
    double x = data[i];
    sum += x*x;
  }
  return sqrt(sum);
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

std::ostream &operator<<(std::ostream &os, const OutputVal &ov) {
  ov.print(os);
  return os;
}

void ScalarOutputVal::print(std::ostream &os) const {
  os << "ScalarOutputVal(" << val << ")";
}

void VectorOutputVal::print(std::ostream &os) const {
  os << "VectorOutputVal(" << data[0];
  for(int i=1; i<size_; i++) {
    os << ", " << data[i];
  }
  os << ")";
}
