# -*- python -*-
# $RCSfile: profilefunction.py,v $
# $Revision: 1.15 $
# $Author: reida $
# $Date: 2005/06/10 18:51:51 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from oof2.common import strfunction
from oof2.common import utils
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump
import struct
import types


# String-based function object used by boundary profiles -- user types
# in a string, and one of these objects gets built.  Profiles'
# __call__ methods provide the relevant arguments.

class ProfileFunction(strfunction.StrFunction):
    def __init__(self, funcstr):
        # x and y are the current location and is guaranteed to be
        # provided. All others are provided if applicable.  nx and ny
        # are the unit-vector components of the boundary normal at
        # this location. i is the node index, s is the arc length,
        # alpha is the fractional arc length.
        strfunction.StrFunction.__init__(
            self, 'x, y, t, nx, ny, i, s, alpha', funcstr)
    def __repr__(self):
        # ProfileFunctions only have to write their values when they
        # are used in ProfileFunctionParameters, which can set
        # themselves from a string. So just write the string.
        return "'%s'" % self.funcstr

xmlmenudump.XMLObjectDoc(
    'ProfileFunction',
    xmlmenudump.loadFile('DISCUSSIONS/engine/object/profilefunction.xml'))

# The ProfileFunctionParameter can accept either a string or a
# ProfileFunction object, but it only stores a ProfileFunction object.
# Like XYStrFunctionParameter, it needs a special "set" function.

class ProfileFunctionParameter(parameter.ParameterBase):
    def __init__(self, name, value=None, default=None, tip=None):
        parameter.ParameterBase.__init__(self, name,
                                         value=value,
                                         default=default, tip=tip)
    def checker(self, x):
        if type(x) is types.StringType:
            return
        if isinstance(x, ProfileFunction):
            return
        parameter.raiseTypeError(type(x), 'string or ProfileFunction')
    def set(self, value):
        if type(value) is types.StringType:
            self.__dict__['value'] = ProfileFunction(value)
        elif isinstance(value, ProfileFunction):
            self.__dict__['value'] = value
        elif value is None:
            self.__dict__['value'] = None
        else:
            raise parameter.ParameterMismatch(
                'Got ' + `value` + ' for Parameter ' + self.name)
    def binaryRepr(self, datafile, value):
        length = len(value.funcstr)
        return struct.pack('>i', length) + value.funcstr
    def binaryRead(self, parser):
        b = parser.getBytes(struct.calcsize('>i'))
        (length,) = struct.unpack('>i', b)
        return ProfileFunction(parser.getBytes(length))
    def valueDesc(self):
        return "A <link linkend='Object:ProfileFunction'><classname>ProfileFunction</classname></link> object."

utils.OOFdefine('ProfileFunction', ProfileFunction)

