// -*- C++ -*-
// $RCSfile: solverdriver.h,v $
// $Revision: 1.7 $
// $Author: langer $
// $Date: 2005/09/02 21:27:44 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#ifndef SOLVERDRIVER_H
#define SOLVERDRIVER_H
class CSRmatrix;
class FEMesh;
class LinearSolver;
class SteppingScheme;

double dot_product (VECTOR_D &left, VECTOR_D &right);

class SolverDriverCore {
  /*
    The base SolverDriverCore class applies a solver 
    to an femesh.
    precompute() creates the different bits, using the FEMesh.
    solve() handles the solver.
    postcompute() places the final solution in the femesh and 
    cleans whatever mess was left behind.
  */

protected:
  LinearSolver * solver;
  FEMesh * femesh;
  SteppingScheme * sscheme;//stepper uses K and b
  //to solve the Kx=b system
  CSRmatrix * K;
  VECTOR_D *b;
  int nrowsA, ncolsA;
public:
  void set_stepping_scheme(SteppingScheme *smethod) {sscheme = smethod;}
  void clear_memory(); 
  CSRmatrix * A;
  CSRmatrix * C;
  VECTOR_D * x;
  VECTOR_D * rhs;
  VECTOR_D * xp;
  int nrows() {return nrowsA;};
  int ncols() {return ncolsA;};
  SolverDriverCore(SteppingScheme *, LinearSolver *, FEMesh *);
  SolverDriverCore() : solver(0),
		       femesh(0),
		       sscheme(0),
		       A(0),
		       C(0),
		       x(0),
		       rhs(0),
		       xp(0),
		       nrowsA(0), 
		       ncolsA(0),
		       K(0),
		       b(0)
		       
  {}
  virtual ~SolverDriverCore();
  virtual double residual();
  virtual void precompute(); // create C-objects that will only
			     // persist during itertative process
  virtual bool solve(); // do real work
  virtual void postcompute(); // clean-up the mess
  void set_solver_components (LinearSolver *ls, FEMesh *fem);
};

class NewtonDriverCore : public SolverDriverCore {
protected:
  VECTOR_D * psi;
  VECTOR_D * dx;
public:
  virtual double residual();
  NewtonDriverCore(SteppingScheme *, LinearSolver *, FEMesh *);
  virtual ~NewtonDriverCore();
  virtual void precompute(); // create C-objects that will persist
			     // during iterative process
  virtual bool solve(); // do real work
};

#endif // SOLVERDRIVER_H
