# -*- python -*-
# $RCSfile: timestepadapter_.py,v $
# $Revision: 1.14 $
# $Author: redwing $
# $Date: 2005/06/15 20:01:09 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.common import cregisteredclass
from oof2.common import debug
from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import xmlmenudump
from oof2.SWIG.common import devel
from oof2.SWIG.common.IO import progressbar

def __get_step_adapter_state(self, meshcontext, steppermethod):
    pbar = progressbar.getProgress()
    steppermethodrequest = (meshcontext.getObject().current_time() >=
                            steppermethod.get_tf()) or pbar.query_stop() > 0
    ## both the steppermethod or the timestepadapter can request to
    ## stop the process
    return not steppermethodrequest and self.keep_going()
TimeStepAdapterPtr.get_step_adapter_state = __get_step_adapter_state

def __TimeStepAdapterIterate(self, meshcontext, steppermethod):
    
    timestepadapterstate = self.get_step_adapter_state(meshcontext, steppermethod)
    if timestepadapterstate:
        ## if we are cool, update time-step using previous time step,
        ## global target time, and error.  set dt for all the
        ## steppermethods, if the specific time step adapter requires
        ## it.
        self.set_time_step(meshcontext.getObject(), steppermethod)
    return timestepadapterstate
TimeStepAdapterPtr.iterate = __TimeStepAdapterIterate

def __TimeStepAdapterApplyAdditional(self, solverdriver, meshcontext,
                                       verbosity = 1, report_success = 1):
    ## default is PASS
    ## subclasses may have other trial solutions to compare against
    ## (interpolations, extrapolations, etc.) which may require to
    ## apply the SAME solverdriver
    pass
TimeStepAdapterPtr.apply_additional_methods = __TimeStepAdapterApplyAdditional

def __TimeStepAdapterApply(self, steppermethod, solverdriver, meshcontext,
                             verbosity = 1, report_success = 1):
        ## solver is applied (maybe more than once), BUT solution is not
    ## accepted yet.
    solverdriver.apply(steppermethod, meshcontext, verbosity)
    self.apply_additional_methods(solverdriver, meshcontext, verbosity,
                                  report_success) ## DEFAULT IS PASS
TimeStepAdapterPtr.apply = __TimeStepAdapterApply

def __TimeStepAdapterUpdate(self, meshcontext, steppermethod):
    residual = self.get_residual(steppermethod)
    ## each stepper adapter returns a residual.
    ## the default is zero.

    ## if residual is within bounds, then accept the solution
    ## otherwise restore the initial state (solution), and adjust the
    ## time step adapter's internal machinery (e.g., time-step)
    ## to try again or bail out from the loop.
    if residual <= self.get_tolerance():
        ## accept solution:
        ## update internal machinery of stepadapter
        ## and current time index
        self.accept(meshcontext.getObject(), steppermethod)
        reporter.report("time index = %g"%meshcontext.getObject().current_time())
    else:
        ## try again:
        ## trash trial solution
        ## restore solution in mesh
        self.reject(meshcontext.getObject())
TimeStepAdapterPtr.update = __TimeStepAdapterUpdate

cregisteredclass.registerCClass(TimeStepAdapterPtr)

TimeStepAdapterPtr.tip = "Methods for dynamically adapting the time-step time-dependent problems."

cregisteredclass.Registration(
    'NullAdapter',
    TimeStepAdapterPtr, ## base class
    NullAdapter, ## subclass
    ordering=100,
    params = [],
    tip = "no time adaptation"
    )

