# -*- python -*-
# $RCSfile: imagemenu.py,v $
# $Revision: 1.56 $
# $Author: reida $
# $Date: 2005/08/05 18:32:19 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.SWIG.common import ooferror
from oof2.SWIG.common.IO import progressbar
from oof2.SWIG.image import oofimage
from oof2.common import debug
from oof2.common import labeltree
from oof2.common import parallel_enable
from oof2.common import quit
from oof2.common import utils
from oof2.common.IO import automatic
from oof2.common.IO import mainmenu
from oof2.common.IO import microstructuremenu
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO import whoville
from oof2.common.IO import xmlmenudump
from oof2.image import imagemodifier
import oof2.common.microstructure
if parallel_enable.query() == 1:
    from oof2.image.IO import oofimageIPC
import os

imagemenu = mainmenu.OOF.addItem(oofmenu.OOFMenuItem(
    'Image',
    cli_only=1,
    help='Operations involving Images.'))

imagemodmenu = mainmenu.OOF.Image.addItem(oofmenu.OOFMenuItem(
    'Modify',
    help='Tools for modifying images.'
    ))

def buildImageModMenu():
    imagemodmenu.clearMenu()
    for registration in imagemodifier.ImageModifier.registry:
        try:
            help = registration.tip
        except AttributeError:
            help = None
        params = [whoville.WhoParameter('image', whoville.getClass('Image'),
                                        tip=parameter.emptyTipString)] \
                 + registration.params
        menuitem = imagemodmenu.addItem(
            oofmenu.OOFMenuItem(registration.name(),
                                callback=imagemodifier.doImageMod,
                                params=params,
                                help=help,
                                discussion=registration.discussion))
        menuitem.data = registration

switchboard.requestCallback(imagemodifier.ImageModifier, buildImageModMenu)
buildImageModMenu()

###################################

def loadImage(menuitem, filename, microstructure, height, width):
    if filename:
        # Read file and create an OOFImage object
        image = autoReadImage(filename, height, width)
        loadImageIntoMS(image, microstructure)

def autoReadImage(filename, height, width):
    kwargs = {}
    if height is not automatic.automatic:
        kwargs['height'] = height
    if width is not automatic.automatic:
        kwargs['width'] = width
    return oofimage.readImage(filename, **kwargs) # OOFImage object
    
        
def loadImageIntoMS(image, microstructure):
    # 'image' is an OOFImage object.
    # 'microstructure' is a Microstructure name.
    
    # See if the Microstructure already exists
    msclass = whoville.getClass('Microstructure')
    try:                            # look for existing microstructure
        ms = msclass[microstructure] # Who object
    except KeyError:
        msobj = oof2.common.microstructure.Microstructure(microstructure,
                                                     image.sizeInPixels(),
                                                     image.size())
        ms = msclass.add(microstructure, msobj)

    # Check size of microstructure
    if ms.getObject().sizeInPixels() != image.sizeInPixels():
        raise ooferror.ErrUserError("Cannot load an image into an existing Microstructure of a different size.")

    # See if the image name is unique in the Microstructure
    newname = oofimage.imageContexts.uniqueName([ms.name(), image.name()])
    image.rename(newname)
    # Create ImageContext object
    imagecontext = oofimage.imageContexts.add([ms.name(),newname], image)
    ms.getObject().addImage(imagecontext)


mainmenu.OOF.File.Load.addItem(
    oofmenu.OOFMenuItem(
    'Image',
    callback=loadImage,
    ellipsis=1,
    params=[parameter.StringParameter('filename', 'image',
                                      tip="Name of the image file."),
            whoville.WhoParameter('microstructure',
                                  whoville.getClass('Microstructure'),
                                  tip=parameter.emptyTipString),
            parameter.AutoNumericParameter('height',
                                           automatic.automatic,
                                           tip= \
                                   "Physical height of image, or 'automatic'."),
            parameter.AutoNumericParameter('width',
                                           automatic.automatic,
                                           tip= \
                                   "Physical width of image, or 'automatic.'")],
    help="Load an Image into an existing Microstructure.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/loadimage.xml')
    ))

def _sensitize(*args, **kwargs):
    if oof2.common.microstructure.microStructures.nActual() == 0:
        mainmenu.OOF.File.Load.Image.disable()
    else:
        mainmenu.OOF.File.Load.Image.enable()

_sensitize()        
switchboard.requestCallback(('new who', 'Microstructure'), _sensitize)
switchboard.requestCallback(('remove who', 'Microstructure'), _sensitize)

##################################

# Use ImageMagick to save an image file.

def saveImage(menuitem, image, filename):
    immidge = oofimage.getImage(image)
    if immidge:
        quit.disallow()
        try:
            immidge.save(filename)
        finally:
            quit.allow()

# mainmenu.OOF.Image.addItem(oofmenu.OOFMenuItem(
mainmenu.OOF.File.Save.addItem(oofmenu.OOFMenuItem(
    'Image',
    callback=saveImage,
    params=[whoville.WhoParameter('image', whoville.getClass('Image'),
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('filename', 'filename', tip="Name of the file.")
            ],
    help="Save an Image in a file.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/saveimage.xml')))

def _fix_image_save(*args):
    if oof2.SWIG.image.oofimage.imageContexts.nActual()==0:
        mainmenu.OOF.File.Save.Image.disable()
    else:
        mainmenu.OOF.File.Save.Image.enable()

switchboard.requestCallback(('new who', 'Image'), _fix_image_save)
switchboard.requestCallback(('remove who', 'Image'), _fix_image_save)
_fix_image_save()
    

##################################

def imageNameResolver(param, startname):
    if param.automatic():
        basename = param.group['image'].value
    else:
        basename = startname
    imagename = labeltree.makePath(basename)[-1]
    msname = labeltree.makePath(param.group['microstructure'].value)[0]
    return whoville.getClass('Image').uniqueName([msname, imagename])

def copyImage(menuitem, image, microstructure, name):
    sourceimage = oofimage.getImage(image)
    immidge = sourceimage.clone(name)
    loadImageIntoMS(immidge, microstructure)

imagemenu.addItem(oofmenu.OOFMenuItem(
    'Copy',
    ellipsis=1,
    callback=copyImage,
    params=parameter.ParameterGroup(
    whoville.WhoParameter('image', whoville.getClass('Image'),
                          tip=parameter.emptyTipString),
    whoville.NewWhoParameter('microstructure',
                             oof2.common.microstructure.microStructures,
                             tip="Microstructure to receive copied image."),
    parameter.AutomaticNameParameter('name',
                                     resolver=imageNameResolver,
                                     value=automatic.automatic,
                                     tip="Name to give to copied image.")
    ),
    help='Copy an Image.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/copyimage.xml')
    ))


def renameImage(menuitem, image, name):
    immidge = oofimage.imageContexts[image]
    immidge.rename(name, exclude=immidge.getObject().name())


imagemenu.addItem(oofmenu.OOFMenuItem(
    'Rename',
    callback=renameImage,
    params=[whoville.WhoParameter('image', whoville.getClass('Image'),
                                  tip=parameter.emptyTipString),
            parameter.StringParameter('name', tip='New name for the image')],
    help="Rename an Image.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/renameimage.xml')
    ))
    
##################################


def deleteImage(menuitem, image):
    immidge = oofimage.imageContexts[image]
    immidge.getMicrostructure().removeImage(immidge)
    whoville.getClass('Image').remove(image)

imagemenu.addItem(oofmenu.OOFMenuItem(
    'Delete',
    callback=deleteImage,
    params=[whoville.WhoParameter('image', whoville.getClass('Image'),
                                  tip=parameter.emptyTipString)
            ],
    help="Delete an Image.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/deleteimage.xml')
    ))
                                      


##################################

def undoImageMod(menuitem, image):
    who = oofimage.imageContexts[image]
    who.begin_writing()
    oofimage.undoModification(image)
    who.end_writing()
    switchboard.notify('modified image', None, image)
    switchboard.notify('redraw')

def redoImageMod(menuitem, image):
    who = oofimage.imageContexts[image]
    who.begin_writing()
    oofimage.redoModification(image)
    who.end_writing()
    switchboard.notify('modified image', None, image)
    switchboard.notify('redraw')

imagemenu.addItem(oofmenu.OOFMenuItem(
    'Undo',
    callback=undoImageMod,
    params=[whoville.WhoParameter('image', whoville.getClass('Image'),
                                  tip=parameter.emptyTipString)],
    help="Undo an Image modification.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/undoimage.xml')
    ))

imagemenu.addItem(oofmenu.OOFMenuItem(
    'Redo',
    callback=redoImageMod,
    params=[
    whoville.WhoParameter('image', whoville.getClass('Image'),
                          tip=parameter.emptyTipString)],
    help="Redo an Image modification.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/redoimage.xml')
    ))

################################

def createPixelGroups(menuitem, image):
    reporter.start_progressbar()
    if parallel_enable.query() == 1:
        from oof2.SWIG.common import mpitools
        if mpitools.rank() == 0:
            oofimageIPC.ipcoofimagemenu.AutoGroup(image = image)
    imagecontext = oofimage.imageContexts[image]
    ms = imagecontext.getMicrostructure()
    mscontext = oof2.common.microstructure.microStructures[ms.name()]
    immidge = imagecontext.getObject()
    pixellistdict = {}
    mscontext.begin_writing()

    pBar = menuitem.getProgressBar("continuous")
    pBar.set_message("Categorizing pixels ...")
    total = ms.sizeInPixels()[0]*ms.sizeInPixels()[1]  # total no. of pixels
    done = 0  # no. of looked-at pixels
    progress = 0.0  # progress
    interrupted = 0  # stop signal
    colornamedict = {}
    try:
        for coord in ms.coords():
            done += 1
            progress = 1.0*done/total
            color = immidge[coord]
            # Don't recreate the group name for each pixel -- it's too slow.
            try:
                colorname = colornamedict[color]
            except KeyError:
                colornamedict[color] = colorname = `color`
            grp = ms.getGroup(colorname)  # the pixel group is being added
            try:
                pixellistdict[grp].append(coord)
            except KeyError:
                pixellistdict[grp] = [coord]

            if pBar.query_stop():  # if stopped by user
                pBar.set_failure()
                pBar.set_message("Failed")
                # when interrupted, added pixel groups have to be removed
                for grp in pixellistdict.keys():
                    ms.removeGroup(grp.name())
                interrupted = 1
                reporter.report("Image auto-grouping interrupted.")
                break
            else:            
                pBar.set_progress(progress)

        if not interrupted:
            pBar.set_message("Creating pixel groups ...")
            for grp, pixellist in pixellistdict.items():
                grp.add(pixellist)
            pBar.set_success()
            pBar.set_message("Succeeded")
                                          
    finally:
        mscontext.end_writing()
        reporter.progressbar_completed()
        reporter.end_progressbar()
        switchboard.notify('redraw')

imagemenu.addItem(oofmenu.OOFMenuItem(
    'AutoGroup',
    callback=createPixelGroups,
    params=[whoville.WhoParameter('image', whoville.getClass('Image'),
                                  tip=parameter.emptyTipString)],
    threadable=oofmenu.THREADABLE,
    help='Create a pixel group for each color in the image.',
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/autogroup.xml')
    ))


###############################

# Create a Microstructure from an existing Image.  Since the Image
# exists, it must be in another Microstructure, so we have to clone
# it and add the clone to the new Microstructure.

# TODO: Currently this operation sets the physical size of the new
# microstructure from the physical size of the old one.  This is a
# sensible default, but it should be possible to specify a new size,
# also.
def createMSFromImage(menuitem, name, image):
    imagepath = labeltree.makePath(image)
    imagecontext = oofimage.imageContexts[image]
    immidge = imagecontext.getObject().clone(imagepath[-1])
    ms = oof2.common.microstructure.Microstructure(name,
                                              immidge.sizeInPixels(),
                                              immidge.size())
    newimagecontext = oofimage.imageContexts.add([name, immidge.name()],
                                                 immidge)
    ms.addImage(newimagecontext)

def msImageNameResolver(param, name):
    if param.automatic():
        imagename = param.group['image'].value
        imagepath = labeltree.makePath(imagename)
        basename = imagepath[-1]
    else:
        basename = name
    return oof2.common.microstructure.microStructures.uniqueName(basename)

microstructuremenu.micromenu.addItem(
    oofmenu.OOFMenuItem(
    'Create_From_Image',
    callback=createMSFromImage,
    params = parameter.ParameterGroup(
    parameter.AutomaticNameParameter('name', msImageNameResolver,
                                     automatic.automatic,
                                     tip="Name of the new Microstructure."),
    whoville.WhoParameter('image', oofimage.imageContexts,
                          tip='Image on which to base the Microstructure.')),
    help="Create a Microstructure from an already loaded Image.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/microfromimage.xml')
    ))

##############################

# Load an Image and create a Microstructure from it in one operation.

def createMSFromImageFile(menuitem, filename, microstructure_name,
                          height, width):
    if (height!=automatic and height<=0) or \
       (width!=automatic and width<=0):
        raise ooferror.ErrUserError(
            "Negative microstructuresizes are not allowed.")
    
    image = autoReadImage(filename, height, width)
    ms = oof2.common.microstructure.Microstructure(microstructure_name,
                                              image.sizeInPixels(),
                                              image.size())
    imagecontext = oofimage.imageContexts.add([ms.name(), image.name()], image)
    ms.addImage(imagecontext)
def msImageFileNameResolver(param, name):
    if param.automatic():
        basename = os.path.basename(param.group['filename'].value)
    else:
        basename = name
    return oof2.common.microstructure.microStructures.uniqueName(basename)

microstructuremenu.micromenu.addItem(oofmenu.OOFMenuItem(
    'Create_From_ImageFile',
    callback=createMSFromImageFile,
    params=parameter.ParameterGroup(
    parameter.StringParameter('filename', tip="Name of the file."),
    parameter.AutomaticNameParameter('microstructure_name',
                                     msImageFileNameResolver,
                                     automatic.automatic,
                                     tip= 'Name of the new Microstructure.'),
    parameter.AutoNumericParameter('height',
                                   automatic.automatic,
                                   tip= \
                                   "Physical height of image, or 'automatic'."),
    parameter.AutoNumericParameter('width',
                                   automatic.automatic,
                                   tip= \
                                   "Physical width of image, or 'automatic'.")),
    help="Load an Image and create a Microstructure from it.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/image/menu/microfromimagefile.xml')
                        ))
